Package: DataDNA
Title: Data Frame Fingerprints and Lineage Figures
Version: 0.1.0
Authors@R: c(
    person("Tony", "Lu", role = c("aut", "cre"),
           email = "xulunt123@gmail.com")
    )
Description: Profiles R data frames as compact data fingerprints using schema,
    shape, missingness, distribution, category, uniqueness, time, and role
    signals. It compares versions, identifies close relatives in a library of
    historical data sets, and renders portable HTML cards plus static PNG/PDF
    lineage figures for reports.
License: MIT + file LICENSE
Encoding: UTF-8
LazyData: true
RoxygenNote: 7.3.2
Depends: R (>= 4.1.0)
Imports: htmltools
NeedsCompilation: no
Packaged: 2026-05-04 10:06:39 UTC; TonyL
Author: Tony Lu [aut, cre]
Maintainer: Tony Lu <xulunt123@gmail.com>
Repository: CRAN
Date/Publication: 2026-05-06 20:00:02 UTC
Built: R 4.5.2; ; 2026-05-06 20:34:43 UTC; unix
