export(data_dna)
export(dna_card)
export(dna_compare)
export(dna_diff)
export(dna_match)
export(dna_match_card)
export(dna_match_plot)
export(dna_species)
export(dna_example_customers)
S3method(print,data_dna)
S3method(print,dna_comparison)
S3method(print,dna_diff)
S3method(print,dna_match)
importFrom(utils,head)
