\name{customers_old}
\alias{customers_old}
\docType{data}
\title{Old customer table for DataDNA examples}
\format{
A data frame with 180 rows and 8 columns.
}
\source{
Synthetic data generated for package examples.
}
\description{
A small synthetic customer table used to demonstrate data DNA profiling.
}
\keyword{datasets}
